% *** ARD-marg_alpha Algorithm ***
% *approach based on analytical marginalization over vector alfa and 
%  following maximizing the marginal likelihood function (min cost fce)
%  to solve set of equations m = Ax + e, where x is sparse vector of unknowns
% *see paper 'Comparison of sparse Bayesian regularization methods
%  for high-dimensional inverse problem'
% *'visualisation of the reconstruction' is prepared for case,
%  where x is n*n image - a modification is easy and possible
%
% % written by Jakub Sevcik and Vaclav Smidl 2016

% Load the measurement matrix (A) and the vector (m)
% *a training dataset is downloadable from http://www.fips.fi/dataset.php
load Data328 A m
m = m(:);

% % ARD-marg_alpha Algorithm
% inicialization
iterations = 20; % number of iteration that use NonLinear Conjugate Gradients method
beta = 1;
E = 1e-2;
Var = 1e-4;

gamma = E^2/Var;
delta = E/Var;

N = size(A,2);
n = sqrt(N);

% minimizing of the cost function
NAtb = norm(A'*m);
f = @(x) 0.5*beta*(A*x - m)'*(A*x - m) + (gamma + 0.5)*sum(log(1 + (x.^2)./(2*delta)));
grad = @(x) beta*A'*(A*x - m) + (2*gamma + 1)*(x./(2*delta + x.^2));

x = nlcgClever(f,grad,N,NAtb,iterations);

% % visualisation of the reconstruction
%%
figure
imagesc(reshape(x,n,n))
colormap gray
axis square
axis off
title(['ARD-marg \alpha Algorithm with ',num2str(iterations),' iterations of NLCG method'])